@@ -57,62 +57,34 @@
 #include <X11/Xpoll.h>
 #include <X11/Xproto.h>
 #include "misc.h"
-
 #include "compiler.h"
-
 #include "xf86.h"
 #include "xf86Priv.h"
 #define XF86_OS_PRIVS
 #include "xf86_OSlib.h"
-#include "atKeynames.h"
-
+#include <X11/keysym.h>
 
 #ifdef XFreeXDGA
 #include "dgaproc.h"
 #endif
 
-#ifdef XINPUT
 #include <X11/extensions/XI.h>
 #include <X11/extensions/XIproto.h>
-#else
 #include "inputstr.h"
-#endif
 #include "xf86Xinput.h"
 
 #include "mi.h"
 #include "mipointer.h"
 
-#ifdef XF86BIGFONT
-#define _XF86BIGFONT_SERVER_
-#include <X11/extensions/xf86bigfont.h>
-#endif
-
-#ifdef XKB
-extern Bool noXkbExtension;
-#endif
+#include "xkbsrv.h"
+#include "xkbstr.h"
 
 #ifdef DPMSExtension
-#define DPMS_SERVER
-#include <X11/extensions/dpms.h>
+#include <X11/extensions/dpmsconst.h>
 #include "dpmsproc.h"
 #endif
 
-#define XE_POINTER  1
-#define XE_KEYBOARD 2
-
-#define EqEnqueue(pDev, ev) { \
-    int __sigstate = xf86BlockSIGIO (); \
-    mieqEnqueue (pDev, ev); \
-    xf86UnblockSIGIO(__sigstate); \
-}
-
 /*
- * The first of many hacks to get VT switching to work under
- * Solaris 2.1 for x86. The basic problem is that Solaris is supposed
- * to be SVR4. It is for the most part, except where the video interface
- * is concerned.  These hacks work around those problems.
- * See the comments for Linux, and SCO.
- *
  * This is a toggling variable:
  *  FALSE = No VT switching keys have been pressed last time around
  *  TRUE  = Possible VT switch Pending
@@ -121,7 +93,7 @@ extern Bool noXkbExtension;
  * This has been generalised to work with Linux and *BSD+syscons (DHD)
  */
 
-_X_EXPORT Bool VTSwitchEnabled = TRUE;	/* Allows run-time disabling for
+Bool VTSwitchEnabled = TRUE;		/* Allows run-time disabling for
                                          *BSD and for avoiding VT
                                          switches when using the DRI
                                          automatic full screen mode.*/
@@ -160,9 +132,8 @@ LegalModifier(unsigned int key, DeviceIntPtr pDev)
  *      Function used for screensaver purposes by the os module. Returns the
  *      time in milliseconds since there last was any input.
  */
-
 int
-TimeSinceLastInputEvent()
+TimeSinceLastInputEvent(void)
 {
   if (xf86Info.lastEventTime == 0) {
     xf86Info.lastEventTime = GetTimeInMillis();
@@ -170,90 +141,34 @@ TimeSinceLastInputEvent()
   return GetTimeInMillis() - xf86Info.lastEventTime;
 }
 
-
-
 /*
  * SetTimeSinceLastInputEvent --
  *      Set the lastEventTime to now.
  */
-
-_X_EXPORT void
-SetTimeSinceLastInputEvent()
+void
+SetTimeSinceLastInputEvent(void)
 {
   xf86Info.lastEventTime = GetTimeInMillis();
 }
 
-
-
 /*
  * ProcessInputEvents --
  *      Retrieve all waiting input events and pass them to DIX in their
  *      correct chronological order. Only reads from the system pointer
  *      and keyboard.
  */
-
 void
-ProcessInputEvents ()
+ProcessInputEvents (void)
 {
   int x, y;
-#ifdef INHERIT_LOCK_STATE
-  static int generation = 0;
-#endif
-
-    /*
-     * With INHERIT_LOCK_STATE defined, the initial state of CapsLock, NumLock
-     * and ScrollLock will be set to match that of the VT the server is
-     * running on.
-     */
-#ifdef INHERIT_LOCK_STATE
-    if (generation != serverGeneration) {
-      xEvent kevent;
-      DevicePtr pKeyboard = xf86Info.pKeyboard;
-      extern unsigned int xf86InitialCaps, xf86InitialNum, xf86InitialScroll;
-
-      generation = serverGeneration;
-      kevent.u.keyButtonPointer.time = GetTimeInMillis();
-      kevent.u.keyButtonPointer.rootX = 0;
-      kevent.u.keyButtonPointer.rootY = 0;
-      kevent.u.u.type = KeyPress;
-
-
-      if (xf86InitialCaps) {
-        kevent.u.u.detail = xf86InitialCaps;
-        (* pKeyboard->processInputProc)(&kevent, (DeviceIntPtr)pKeyboard, 1);
-        xf86InitialCaps = 0;
-      }
-      if (xf86InitialNum) {
-        kevent.u.u.detail = xf86InitialNum;
-        (* pKeyboard->processInputProc)(&kevent, (DeviceIntPtr)pKeyboard, 1);
-        xf86InitialNum = 0;
-      }
-      if (xf86InitialScroll) {
-        kevent.u.u.detail = xf86InitialScroll;
-        (* pKeyboard->processInputProc)(&kevent, (DeviceIntPtr)pKeyboard, 1);
-        xf86InitialScroll = 0;
-      }
-    }
-#endif
-
-  xf86Info.inputPending = FALSE;
 
   mieqProcessInputEvents();
-  miPointerUpdateSprite(inputInfo.pointer);
 
+  /* FIXME: This is a problem if we have multiple pointers */
   miPointerGetPosition(inputInfo.pointer, &x, &y);
   xf86SetViewport(xf86Info.currentScreen, x, y);
 }
 
-void
-xf86GrabServerCallback(CallbackListPtr *callbacks, pointer data, pointer args)
-{
-    ServerGrabInfoRec *grab = (ServerGrabInfoRec*)args;
-
-    xf86Info.grabInfo.server.client = grab->client;
-    xf86Info.grabInfo.server.grabstate = grab->grabstate;
-}
-
 /*
  * Handle keyboard events that cause some kind of "action"
  * (i.e., server termination, video mode changes, VT switches, etc.)
@@ -261,9 +176,7 @@ xf86GrabServerCallback(CallbackListPtr *callbacks, pointer data, pointer args)
 void
 xf86ProcessActionEvent(ActionEvent action, void *arg)
 {
-#ifdef DEBUG
-    ErrorF("ProcessActionEvent(%d,%x)\n", (int) action, arg);
-#endif
+    DebugF("ProcessActionEvent(%d,%x)\n", (int) action, arg);
     switch (action) {
     case ACTION_TERMINATE:
 	if (!xf86Info.dontZap) {
@@ -281,51 +194,19 @@ xf86ProcessActionEvent(ActionEvent action, void *arg)
 	if (!xf86Info.dontZoom)
 	    xf86ZoomViewport(xf86Info.currentScreen, -1);
 	break;
-    case ACTION_DISABLEGRAB:
-	if (!xf86Info.grabInfo.disabled && xf86Info.grabInfo.allowDeactivate) {
-	  if (inputInfo.pointer && inputInfo.pointer->grab != NULL &&
-	      inputInfo.pointer->DeactivateGrab)
-	    inputInfo.pointer->DeactivateGrab(inputInfo.pointer);
-	  if (inputInfo.keyboard && inputInfo.keyboard->grab != NULL &&
-	      inputInfo.keyboard->DeactivateGrab)
-	    inputInfo.keyboard->DeactivateGrab(inputInfo.keyboard);
-	}
-	break;
-    case ACTION_CLOSECLIENT:
-	if (!xf86Info.grabInfo.disabled && xf86Info.grabInfo.allowClosedown) {
-	  ClientPtr pointer, keyboard, server;
-
-	  pointer = keyboard = server = NULL;
-	  if (inputInfo.pointer && inputInfo.pointer->grab != NULL)
-	    pointer = clients[CLIENT_ID(inputInfo.pointer->grab->resource)];
-	  if (inputInfo.keyboard && inputInfo.keyboard->grab != NULL) {
-	    keyboard = clients[CLIENT_ID(inputInfo.keyboard->grab->resource)];
-	    if (keyboard == pointer)
-	      keyboard = NULL;
-	  }
-	  if ((xf86Info.grabInfo.server.grabstate == SERVER_GRABBED) &&
-	      (((server = xf86Info.grabInfo.server.client) == pointer) ||
-	       (server == keyboard)))
-	      server = NULL;
-
-	  if (pointer)
-	    CloseDownClient(pointer);
-	  if (keyboard)
-	    CloseDownClient(keyboard);
-	  if (server)
-	    CloseDownClient(server);
-	}
-	break;
-#if !defined(__SOL8__) && !defined(sgi) && \
-    (!defined(sun) || defined(__i386__)) && defined(VT_ACTIVATE)
+#if defined(VT_ACTIVATE)
     case ACTION_SWITCHSCREEN:
 	if (VTSwitchEnabled && !xf86Info.dontVTSwitch && arg) {
 	    int vtno = *((int *) arg);
 #if defined(__SCO__) || defined(__UNIXWARE__)
 	    vtno--;
 #endif
-#if defined(QNX4)
-	    xf86Info.vtRequestsPending = vtno;
+#if defined(sun)
+	    if (vtno == xf86Info.vtno)
+		break;
+
+	    xf86Info.vtRequestsPending = TRUE;
+	    xf86Info.vtPendingNum = vtno;
 #else
 	    if (ioctl(xf86Info.consoleFd, VT_ACTIVATE, vtno) < 0)
 		ErrorF("Failed to switch consoles (%s)\n", strerror(errno));
@@ -334,7 +215,6 @@ xf86ProcessActionEvent(ActionEvent action, void *arg)
 	break;
     case ACTION_SWITCHSCREEN_NEXT:
 	if (VTSwitchEnabled && !xf86Info.dontVTSwitch) {
-/* Shouldn't this be true for (sun) && (i386) && (SVR4) ? */
 #if defined(__SCO__) || defined(__UNIXWARE__)
 	    if (ioctl(xf86Info.consoleFd, VT_ACTIVATE, xf86Info.vtno) < 0)
 #else
@@ -355,78 +235,11 @@ xf86ProcessActionEvent(ActionEvent action, void *arg)
 	}
 	break;
 #endif
-    case ACTION_MESSAGE:
-        {
-            char *retstr, *message = (char *) arg;
-	    ScrnInfoPtr pScr = XF86SCRNINFO(xf86Info.currentScreen);
-
-#ifdef DEBUG
-            ErrorF("ActionMessage: '%s'\n", message);
-#endif
-	    /* Okay the message made it to the ddx.  The common layer */
-	    /* can check for relevant messages here and react to any  */
-	    /* that have a global effect.  For example:               */
-	    /*                                                        */
-	    /* if (!strcmp(message, "foo") {                          */
-	    /*      do_foo(); break                                   */
-	    /* }                                                      */
-	    /*                                                        */
-	    /* otherwise fallback to sending a key event message to   */
-	    /* the current screen's driver:                           */
-	    if (*pScr->HandleMessage != NULL) {
-		(void) (*pScr->HandleMessage)(pScr->scrnIndex,
-			"KeyEventMessage", message, &retstr);
-	    }
-        }
-	break;
     default:
 	break;
     }
 }
 
-#define ModifierIsSet(k) ((modifiers & (k)) == (k))
-
-_X_EXPORT Bool
-xf86CommonSpecialKey(int key, Bool down, int modifiers)
-{
-  if ((!ModifierIsSet(ShiftMask)) &&
-      (((ModifierIsSet(ControlMask | AltMask)) ||
-        (ModifierIsSet(ControlMask | AltLangMask))))) {
-      switch (key) {
-	
-      case KEY_BackSpace:
-	xf86ProcessActionEvent(ACTION_TERMINATE, NULL);
-	break;
-
-      /*
-       * Check grabs
-       */
-      case KEY_KP_Divide:
-	xf86ProcessActionEvent(ACTION_DISABLEGRAB, NULL);
-	break;
-      case KEY_KP_Multiply:
-	xf86ProcessActionEvent(ACTION_CLOSECLIENT, NULL);
-	break;
-	
-	/*
-	 * The idea here is to pass the scancode down to a list of
-	 * registered routines. There should be some standard conventions
-	 * for processing certain keys.
-	 */
-      case KEY_KP_Minus:   /* Keypad - */
-	if (down) xf86ProcessActionEvent(ACTION_PREV_MODE, NULL);
-	if (!xf86Info.dontZoom) return TRUE;
-	break;
-	
-      case KEY_KP_Plus:   /* Keypad + */
-	if (down) xf86ProcessActionEvent(ACTION_NEXT_MODE, NULL);
-	if (!xf86Info.dontZoom) return TRUE;
-	break;
-      }
-  }
-  return FALSE;
-}
-
 /*
  * xf86Wakeup --
  *      Os wakeup handler.
@@ -436,7 +249,6 @@ xf86CommonSpecialKey(int key, Bool down, int modifiers)
 void
 xf86Wakeup(pointer blockData, int err, pointer pReadmask)
 {
-#if !defined(__QNX__)
     fd_set* LastSelectMask = (fd_set*)pReadmask;
     fd_set devicesWithInput;
     InputInfoPtr pInfo;
@@ -451,39 +263,19 @@ xf86Wakeup(pointer blockData, int err, pointer pReadmask)
 		    (FD_ISSET(pInfo->fd, &devicesWithInput) != 0)) {
 		    int sigstate = xf86BlockSIGIO();
 
-		    pInfo->read_input(pInfo);
-		    xf86UnblockSIGIO(sigstate);
 		    /*
 		     * Remove the descriptior from the set because more than one
 		     * device may share the same file descriptor.
 		     */
 		    FD_CLR(pInfo->fd, &devicesWithInput);
+
+		    pInfo->read_input(pInfo);
+		    xf86UnblockSIGIO(sigstate);
 		}
 		pInfo = pInfo->next;
 	    }
 	}
     }
-#else   /* __QNX__ */
-
-    InputInfoPtr pInfo;
-
-    pInfo = xf86InputDevs;
-    while (pInfo) {
-		if (pInfo->read_input && pInfo->fd >= 0) {
-		    int sigstate = xf86BlockSIGIO();
-
-		    pInfo->read_input(pInfo);
-		    xf86UnblockSIGIO(sigstate);
-		    /*
-		     * Must break here because more than one device may share
-		     * the same file descriptor.
-		     */
-		    break;
-		}
-		pInfo = pInfo->next;
-    }
-
-#endif  /* __QNX__ */
 
     if (err >= 0) { /* we don't want the handlers called if select() */
 	IHPtr ih;   /* returned with an error condition, do we?      */
@@ -497,8 +289,6 @@ xf86Wakeup(pointer blockData, int err, pointer pReadmask)
     }
 
     if (xf86VTSwitchPending()) xf86VTSwitch();
-
-    if (xf86Info.inputPending) ProcessInputEvents();
 }
 
 
@@ -507,16 +297,13 @@ xf86Wakeup(pointer blockData, int err, pointer pReadmask)
  *    signal handler for the SIGIO signal.
  */
 static void
-xf86SigioReadInput(int fd,
-		   void *closure)
+xf86SigioReadInput(int fd, void *closure)
 {
     int errno_save = errno;
-    int sigstate = xf86BlockSIGIO();
-    InputInfoPtr pInfo = (InputInfoPtr) closure;
+    InputInfoPtr pInfo = closure;
 
     pInfo->read_input(pInfo);
 
-    xf86UnblockSIGIO(sigstate);
     errno = errno_save;
 }
 
@@ -524,7 +311,7 @@ xf86SigioReadInput(int fd,
  * xf86AddEnabledDevice --
  *
  */
-_X_EXPORT void
+void
 xf86AddEnabledDevice(InputInfoPtr pInfo)
 {
     if (!xf86InstallSIGIOHandler (pInfo->fd, xf86SigioReadInput, pInfo)) {
@@ -536,7 +323,7 @@ xf86AddEnabledDevice(InputInfoPtr pInfo)
  * xf86RemoveEnabledDevice --
  *
  */
-_X_EXPORT void
+void
 xf86RemoveEnabledDevice(InputInfoPtr pInfo)
 {
     if (!xf86RemoveSIGIOHandler (pInfo->fd)) {
@@ -546,7 +333,7 @@ xf86RemoveEnabledDevice(InputInfoPtr pInfo)
 
 static int *xf86SignalIntercept = NULL;
 
-_X_EXPORT void
+void
 xf86InterceptSignals(int *signo)
 {
     if ((xf86SignalIntercept = signo))
@@ -555,57 +342,55 @@ xf86InterceptSignals(int *signo)
 
 static void (*xf86SigIllHandler)(void) = NULL;
 
-_X_EXPORT void
+void
 xf86InterceptSigIll(void (*sigillhandler)(void))
 {
     xf86SigIllHandler = sigillhandler;
 }
 
 /*
- * xf86SigHandler --
+ * xf86SigWrapper --
  *    Catch unexpected signals and exit or continue cleanly.
  */
-void
-xf86SigHandler(int signo)
+int
+xf86SigWrapper(int signo)
 {
   if ((signo == SIGILL) && xf86SigIllHandler) {
     (*xf86SigIllHandler)();
-    /* Re-arm handler just in case we unexpectedly return here */
-    (void) signal(signo, xf86SigHandler);
-    return;
+    return 0; /* continue */
   }
 
   if (xf86SignalIntercept && (*xf86SignalIntercept < 0)) {
     *xf86SignalIntercept = signo;
-    /* Re-arm handler just in case */
-    (void) signal(signo, xf86SigHandler);
-    return;
+    return 0; /* continue */
   }
 
-  signal(signo,SIG_IGN);
   xf86Info.caughtSignal = TRUE;
-#ifdef XF86BIGFONT
-  XF86BigfontCleanup();
-#endif
-
-  xorg_backtrace();
-
-  FatalError("Caught signal %d.  Server aborting\n", signo);
+  return 1; /* abort */
 }
 
+/*
+ * xf86PrintBacktrace --
+ *    Print a stack backtrace for debugging purposes.
+ */
+void
+xf86PrintBacktrace(void)
+{
+    xorg_backtrace();
+}
+
+#define KeyPressed(k) (keyc->postdown[k >> 3] & (1 << (k & 7)))
+
 static void
 xf86ReleaseKeys(DeviceIntPtr pDev)
 {
-    KeyClassPtr keyc = NULL;
-    KeySym *map = NULL;
-    xEvent ke;
-    int i = 0, j = 0, nevents = 0;
+    KeyClassPtr keyc;
+    int i, j, nevents, sigstate;
 
     if (!pDev || !pDev->key)
         return;
 
     keyc = pDev->key;
-    map = keyc->curKeySyms.map;
 
     /*
      * Hmm... here is the biggest hack of every time !
@@ -618,36 +403,15 @@ xf86ReleaseKeys(DeviceIntPtr pDev)
      * are reenabled.
      */
 
-    for (i = keyc->curKeySyms.minKeyCode, map = keyc->curKeySyms.map;
-         i < keyc->curKeySyms.maxKeyCode;
-         i++, map += keyc->curKeySyms.mapWidth) {
+    for (i = keyc->xkbInfo->desc->min_key_code;
+         i < keyc->xkbInfo->desc->max_key_code;
+         i++) {
         if (KeyPressed(i)) {
-            switch (*map) {
-            /* Don't release the lock keys */
-            case XK_Caps_Lock:
-            case XK_Shift_Lock:
-            case XK_Num_Lock:
-            case XK_Scroll_Lock:
-            case XK_Kana_Lock:
-                break;
-            default:
-                if (pDev == inputInfo.keyboard) {
-                    ke.u.keyButtonPointer.time = GetTimeInMillis();
-                    ke.u.keyButtonPointer.rootX = 0;
-                    ke.u.keyButtonPointer.rootY = 0;
-                    ke.u.u.type = KeyRelease;
-                    ke.u.u.detail = i;
-                    (*pDev->public.processInputProc) (&ke, pDev, 1);
-                }
-                else {
-		    int sigstate = xf86BlockSIGIO ();
-                    nevents = GetKeyboardEvents(xf86Events, pDev, KeyRelease, i);
-                    for (j = 0; j < nevents; j++)
-                        mieqEnqueue(pDev, xf86Events + j);
-		    xf86UnblockSIGIO(sigstate);
-                }
-                break;
-            }
+            sigstate = xf86BlockSIGIO ();
+            nevents = GetKeyboardEvents(xf86Events, pDev, KeyRelease, i);
+            for (j = 0; j < nevents; j++)
+                mieqEnqueue(pDev, (InternalEvent*)(xf86Events + j)->event);
+            xf86UnblockSIGIO(sigstate);
         }
     }
 }
@@ -657,15 +421,13 @@ xf86ReleaseKeys(DeviceIntPtr pDev)
  *      Handle requests for switching the vt.
  */
 static void
-xf86VTSwitch()
+xf86VTSwitch(void)
 {
   int i, prevSIGIO;
   InputInfoPtr pInfo;
   IHPtr ih;
 
-#ifdef DEBUG
-  ErrorF("xf86VTSwitch()\n");
-#endif
+  DebugF("xf86VTSwitch()\n");
 
 #ifdef XFreeXDGA
   if(!DGAVTSwitch())
@@ -678,10 +440,8 @@ xf86VTSwitch()
    */
   if (xf86Screens[0]->vtSema) {
 
-#ifdef DEBUG
-    ErrorF("xf86VTSwitch: Leaving, xf86Exiting is %s\n",
+    DebugF("xf86VTSwitch: Leaving, xf86Exiting is %s\n",
 	   BOOLTOSTRING((dispatchException & DE_TERMINATE) ? TRUE : FALSE));
-#endif
 #ifdef DPMSExtension
     if (DPMSPowerLevel != DPMSModeOn)
 	DPMSSet(serverClient, DPMSModeOn);
@@ -696,29 +456,27 @@ xf86VTSwitch()
      * Keep the order: Disable Device > LeaveVT
      *                        EnterVT > EnableDevice
      */
-    pInfo = xf86InputDevs;
-    while (pInfo) {
-      if (pInfo->dev)
-          DisableDevice(pInfo->dev);
-      pInfo = pInfo->next;
+    for (ih = InputHandlers; ih; ih = ih->next)
+      xf86DisableInputHandler(ih);
+    for (pInfo = xf86InputDevs; pInfo; pInfo = pInfo->next) {
+      if (pInfo->dev) {
+          xf86ReleaseKeys(pInfo->dev);
+          ProcessInputEvents();
+          DisableDevice(pInfo->dev, TRUE);
+      }
     }
     xf86EnterServerState(SETUP);
     for (i = 0; i < xf86NumScreens; i++)
 	xf86Screens[i]->LeaveVT(i, 0);
 
-    for (ih = InputHandlers; ih; ih = ih->next)
-      xf86DisableInputHandler(ih);
     xf86AccessLeave();      /* We need this here, otherwise */
-    xf86AccessLeaveState(); /* console won't be restored    */
 
     if (!xf86VTSwitchAway()) {
       /*
        * switch failed
        */
 
-#ifdef DEBUG
-      ErrorF("xf86VTSwitch: Leave failed\n");
-#endif
+      DebugF("xf86VTSwitch: Leave failed\n");
       prevSIGIO = xf86BlockSIGIO();
       xf86AccessEnter();
       xf86EnterServerState(SETUP);
@@ -737,14 +495,10 @@ xf86VTSwitch()
 
       pInfo = xf86InputDevs;
       while (pInfo) {
-        if (pInfo->dev) {
-            xf86ReleaseKeys(pInfo->dev);
-            EnableDevice(pInfo->dev);
-        }
+        if (pInfo->dev)
+            EnableDevice(pInfo->dev, TRUE);
 	pInfo = pInfo->next;
       }
-      /* XXX HACK */
-      xf86ReleaseKeys(inputInfo.keyboard);
       for (ih = InputHandlers; ih; ih = ih->next)
         xf86EnableInputHandler(ih);
 
@@ -763,17 +517,13 @@ xf86VTSwitch()
  	     * trap calls when switched away.
  	     */
 	    xf86Screens[i]->vtSema = FALSE;
-	    xf86Screens[i]->access = NULL;
-	    xf86Screens[i]->busAccess = NULL;
 	}
 	if (xorgHWAccess)
 	    xf86DisableIO();
     }
   } else {
 
-#ifdef DEBUG
-    ErrorF("xf86VTSwitch: Entering\n");
-#endif
+    DebugF("xf86VTSwitch: Entering\n");
     if (!xf86VTSwitchTo()) return;
 
     prevSIGIO = xf86BlockSIGIO();
@@ -801,14 +551,10 @@ xf86VTSwitch()
 
     pInfo = xf86InputDevs;
     while (pInfo) {
-      if (pInfo->dev) {
-          xf86ReleaseKeys(pInfo->dev);
-          EnableDevice(pInfo->dev);
-      }
+      if (pInfo->dev)
+          EnableDevice(pInfo->dev, TRUE);
       pInfo = pInfo->next;
     }
-    /* XXX HACK */
-    xf86ReleaseKeys(inputInfo.keyboard);
 
     for (ih = InputHandlers; ih; ih = ih->next)
       xf86EnableInputHandler(ih);
@@ -843,7 +589,7 @@ addInputHandler(int fd, InputHandlerProc proc, pointer data)
     return ih;
 }
 
-_X_EXPORT pointer
+pointer
 xf86AddInputHandler(int fd, InputHandlerProc proc, pointer data)
 {
     IHPtr ih = addInputHandler(fd, proc, data);
@@ -853,7 +599,7 @@ xf86AddInputHandler(int fd, InputHandlerProc proc, pointer data)
     return ih;
 }
 
-_X_EXPORT pointer
+pointer
 xf86AddGeneralHandler(int fd, InputHandlerProc proc, pointer data)
 {
     IHPtr ih = addInputHandler(fd, proc, data);
@@ -880,7 +626,7 @@ removeInputHandler(IHPtr ih)
     xfree(ih);
 }
 
-_X_EXPORT int
+int
 xf86RemoveInputHandler(pointer handler)
 {
     IHPtr ih;
@@ -899,7 +645,7 @@ xf86RemoveInputHandler(pointer handler)
     return fd;
 }
 
-_X_EXPORT int
+int
 xf86RemoveGeneralHandler(pointer handler)
 {
     IHPtr ih;
@@ -918,7 +664,7 @@ xf86RemoveGeneralHandler(pointer handler)
     return fd;
 }
 
-_X_EXPORT void
+void
 xf86DisableInputHandler(pointer handler)
 {
     IHPtr ih;
@@ -932,7 +678,7 @@ xf86DisableInputHandler(pointer handler)
 	RemoveEnabledDevice(ih->fd);
 }
 
-_X_EXPORT void
+void
 xf86DisableGeneralHandler(pointer handler)
 {
     IHPtr ih;
@@ -946,7 +692,7 @@ xf86DisableGeneralHandler(pointer handler)
 	RemoveGeneralSocket(ih->fd);
 }
 
-_X_EXPORT void
+void
 xf86EnableInputHandler(pointer handler)
 {
     IHPtr ih;
@@ -960,7 +706,7 @@ xf86EnableInputHandler(pointer handler)
 	AddEnabledDevice(ih->fd);
 }
 
-_X_EXPORT void
+void
 xf86EnableGeneralHandler(pointer handler)
 {
     IHPtr ih;
@@ -977,7 +723,7 @@ xf86EnableGeneralHandler(pointer handler)
 /*
  * As used currently by the DRI, the return value is ignored.
  */
-_X_EXPORT Bool
+Bool
 xf86EnableVTSwitch(Bool new)
 {
     static Bool def = TRUE;
@@ -996,23 +742,6 @@ xf86EnableVTSwitch(Bool new)
 }
 
 void
-xf86ReloadInputDevs(int sig)
-{
-  InputInfoPtr pInfo;
-
-  signal(sig, (void(*)(int))xf86ReloadInputDevs);
-
-  pInfo = xf86InputDevs;
-  while (pInfo) {
-    DisableDevice(pInfo->dev);
-    EnableDevice(pInfo->dev);
-    pInfo = pInfo->next;
-  }
-
-  return;
-}
-
-_X_EXPORT void
 DDXRingBell(int volume, int pitch, int duration) {
     xf86OSRingBell(volume, pitch, duration);
 }
